@@ -1,6 +1,6 @@
 package Bio::Graphics::Glyph;
 
-# $Id: Glyph.pm,v 1.113.4.5 2006/11/01 21:25:44 lstein Exp $
+# $Id: Glyph.pm,v 1.113.4.7 2006/11/23 00:40:02 lstein Exp $
 
 use strict;
 use Carp 'croak','cluck';
@@ -124,12 +124,7 @@ sub feature_has_subparts {
 
   return $self->{feature_has_subparts} = shift if @_;
   return 0 if $self->maxdepth == 0;
-
-  # $feature->compound is an artefact from aggregators. Sadly, an aggregated feature can miss
-  # parts that are out of the query range - this is a horrible feature. Aggregated features have
-  # a compound flag to hack around this.
-  my $feature = $self->feature;
-  $self->{feature_has_subparts} || ($feature->can('compound') && $feature->compound)
+  return $self->{feature_has_subparts};
 }
 
 sub feature { shift->{feature} }
@@ -194,11 +189,15 @@ sub add_feature {
   my $self       = shift;
   my $factory    = $self->factory;
 
+  my $filter     = $self->code_option('filter');
+  my $iscode     = $filter && ref($filter) eq 'CODE';
+
   for my $feature (@_) {
     if (ref $feature eq 'ARRAY') {
       $self->add_group(@$feature);
     } else {
       warn $factory if DEBUG;
+      next if $iscode && !$filter->($feature);
       push @{$self->{parts}},$factory->make_glyph(0,$feature);
     }
   }
@@ -1134,7 +1133,17 @@ sub no_subparts {
 
 sub maxdepth {
   my $self = shift;
-  return $self->option('maxdepth');
+
+  my $maxdepth =  $self->option('maxdepth');
+  return $maxdepth if defined $maxdepth;
+
+  # $feature->compound is an artefact from aggregators. Sadly, an aggregated feature can miss
+  # parts that are out of the query range - this is a horrible mis-feature. Aggregated features have
+  # a compound flag to hack around this.
+  my $feature = $self->feature;
+  return 1 if $feature->can('compound') && $feature->compound;
+
+  return;
 }
 
 sub exceeds_depth {
@@ -1629,6 +1638,10 @@ glyph pages for more options.
                These options are used when creating imagemaps
                for display on the web.  See L<Bio::Graphics::Panel/"Creating Imagemaps">.
 
+  -filter      Select which features to
+               display. Must be a CODE reference.
+
+
 For glyphs that consist of multiple segments, the B<-connector> option
 controls what's drawn between the segments.  The default is undef (no
 connector).  Options include:
@@ -1733,6 +1746,11 @@ The B<-no_subparts> option will prevent the glyph from searching its
 feature for subfeatures. This may enhance performance if you know in
 advance that none of your features contain subfeatures.
 
+The B<-filter> option, which must be a CODE reference, will be invoked
+once for each feature prior to rendering it. The coderef will receive
+the feature as its single option and should return true if the feature
+is to be shown and false otherwise.
+
 =head1 SUBCLASSING Bio::Graphics::Glyph
 
 By convention, subclasses are all lower-case.  Begin each subclass
